package io.tswf.transmutations.common.reader

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.exception.InvalidFieldDescriptionAnnotationException
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AnnotationNode

@CompileStatic
class FieldDescriptionReader {

    public static final String CLASS_REFERENCE_PARAMETER_NAME = "type"
    public static final String DESCRIPTOR_PARAMETER_NAME = "desc"
    public static final String NAME_PARAMETER_NAME = "name"

    static Tuple2<String, Type> findFieldDescriptionFromAnnotation(AnnotationNode annotationNode) {
        def classRef = annotationNode.findValueByName(CLASS_REFERENCE_PARAMETER_NAME, AnnotationNode)
        def descriptor = annotationNode.findValueByName(DESCRIPTOR_PARAMETER_NAME, String)
        def name = annotationNode.getValueByName(NAME_PARAMETER_NAME, String)

        if (classRef != null && descriptor != null) {
            throw new InvalidFieldDescriptionAnnotationException("Parameters ${DESCRIPTOR_PARAMETER_NAME} and ${CLASS_REFERENCE_PARAMETER_NAME} can not be used together in @${Type.getType(annotationNode.desc).className}!")
        }

        def type

        if (classRef != null) {
            type = ClassReferenceReader.getTypeFromClassRefAnnotation(classRef)
        } else {
            type = Type.getType(descriptor)
        }

        return new Tuple2<String, Type>(name, type)
    }

}
